from schema import create_session
from workspaces.capability_interfaces import CapabilityInfoIF
from workspaces.schema import Workflow
from workspaces.services import WorkflowInfo, CapabilityInfo
from workspaces.workflow_interfaces import WorkflowInfoIF


SESSION = create_session("SDM")


def get_workflow_info() -> WorkflowInfoIF:
    """Return a WorkflowInfo based on a real database connection"""
    return WorkflowInfo(SESSION)


def get_capability_info() -> CapabilityInfoIF:
    """Return a CapabilityInfo based on a real database connection"""
    return CapabilityInfo(SESSION)


def test_get_requests_for_capability():
    """Ensure we can get all requests for a certain capability"""
    info = get_capability_info()
    info.create_capability("test_get_requests", None, 0)
    info.create_capability_request("test_get_requests")
    assert len(info.requests_for_capability("test_get_requests")) > 0


def test_persisting_workflow_request():
    """Ensure we can persist a workflow request"""
    info = get_workflow_info()
    null = info.lookup_workflow_definition("null")
    req = info.create_workflow_request(null, {})
    assert req.workflow_request_id is not None


def test_reading_workflows():
    """Ensure we can get workflows"""
    info = get_workflow_info()
    workflows = info.all_workflows()
    assert len(workflows) > 0


def test_reading_workflow_requests():
    """Ensure we can get workflow requests"""
    info = get_workflow_info()
    workflow_requests = info.all_workflow_requests()
    print(workflow_requests)
    assert len(workflow_requests) > 0


def test_reading_capability():
    """Ensure we can get a capability"""
    info = get_capability_info()
    null = info.lookup_capability("null")
    assert null is not None


def test_capability_associated_to_workflow():
    cinfo = get_capability_info()
    winfo = get_workflow_info()
    req = cinfo.create_capability_request("null", {})
    exec = cinfo.create_execution(req)
    workflow_req = winfo.create_workflow_request(
        winfo.lookup_workflow_definition("null"), {}
    )
    exec.current_workflow_request = workflow_req
    SESSION.add(exec)
